﻿/*
Copyright (C) 2014 Tomasz Romaszkiewicz
 
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation 
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, 
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the 
Software is furnished to do so.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;
using System.Threading;
using System.Threading.Tasks;

namespace Example
{
    public partial class MainPage : PhoneApplicationPage
    {
        public MainPage()
        {
            InitializeComponent();
            FirstBox.TextChanged += Box_ChangedFour;
            SecondBox.TextChanged += Box_ChangedFour;
            ThirdBox.TextChanged += Box_ChangedFour;

            mySlider.ValueChanged += mySlider_ValueChanged;
        }

        private string info = "";
        private void Box_ChangedOne(object sender, TextChangedEventArgs e)
        {
            TextBox modifedTextBox = sender as TextBox;
            if (modifedTextBox.Name == "FirstBox") info = "User modified FirstBox";
            else if (modifedTextBox.Name == "SecondBox") info = "User modified SecondBox";
            else if (modifedTextBox.Name == "ThirdBox") info = "User modified ThirdBox";
            FirstBox.Text = info; // this line fires next Box_ChangedOne event
            SecondBox.Text = info; // this line fires next Box_ChangedOne event
            ThirdBox.Text = info; // this line fires next Box_ChangedOne event
        }

        private bool dontChangeFlag = false;
        private void Box_ChangedTwo(object sender, TextChangedEventArgs e)
        {
            if (!dontChangeFlag)
            {
                TextBox modifedTextBox = sender as TextBox;
                if (modifedTextBox.Name == "FirstBox") info = "User modified FirstBox";
                else if (modifedTextBox.Name == "SecondBox") info = "User modified SecondBox";
                else if (modifedTextBox.Name == "ThirdBox") info = "User modified ThirdBox";
                dontChangeFlag = true; // set the lag to prevent next fired event from further changing Boxes
                FirstBox.Text = info;
                SecondBox.Text = info;
                ThirdBox.Text = info;
                dontChangeFlag = false; // enable changing
            }
        }

        private async void Box_ChangedThree(object sender, TextChangedEventArgs e)
        {
            if (!dontChangeFlag)
            {
                TextBox modifedTextBox = sender as TextBox;
                if (modifedTextBox.Name == "FirstBox") info = "User modified FirstBox";
                else if (modifedTextBox.Name == "SecondBox") info = "User modified SecondBox";
                else if (modifedTextBox.Name == "ThirdBox") info = "User modified ThirdBox";
                dontChangeFlag = true; // set the lag to prevent next fired event from further changing Boxes
                FirstBox.Text = info;
                SecondBox.Text = info;
                ThirdBox.Text = info;
                await Task.Delay(1000); // let's give some time to change Boxes
                dontChangeFlag = false; // enable changing
            }
        }

        private string enteredText = "";
        private SemaphoreSlim semWait = new SemaphoreSlim(0, 1);
        private async void Box_ChangedFour(object sender, TextChangedEventArgs e)
        {
            if (!dontChangeFlag)
            {
                TextBox modifedTextBox = sender as TextBox;
                if (modifedTextBox.Name == "FirstBox")
                {
                    info = "User modified FirstBox";
                    enteredText = FirstBox.Text;
                }
                else if (modifedTextBox.Name == "SecondBox") info = "User modified SecondBox";
                else if (modifedTextBox.Name == "ThirdBox") info = "User modified ThirdBox";
                dontChangeFlag = true; // set the lag to prevent next fired event from further changing Boxes
                FirstBox.Text = enteredText;
                await semWait.WaitAsync(); // wait until finished changing
                SecondBox.Text = info;
                await semWait.WaitAsync(); // wait until finished changing
                ThirdBox.Text = info;
                await semWait.WaitAsync(); // wait until finished changing
                dontChangeFlag = false; // enable changing
            }
            else semWait.Release();
        }

        private async void Box_ChangedFive(object sender, TextChangedEventArgs e)
        {
            if (!dontChangeFlag)
            {
                TextBox modifedTextBox = sender as TextBox;
                if (modifedTextBox.Name == "FirstBox")
                {
                    info = "User modified FirstBox";
                    enteredText = FirstBox.Text;
                }
                else if (modifedTextBox.Name == "SecondBox") info = "User modified SecondBox";
                else if (modifedTextBox.Name == "ThirdBox") info = "User modified ThirdBox";
                dontChangeFlag = true; // set the lag to prevent next fired event from further changing Boxes
                if (FirstBox.Text != enteredText)
                {
                    FirstBox.Text = info;
                    await semWait.WaitAsync(); // wait until finished changing
                }
                if (SecondBox.Text != info)
                {
                    SecondBox.Text = info;
                    await semWait.WaitAsync(); // wait until finished changing
                }
                if (ThirdBox.Text != info)
                {
                    ThirdBox.Text = info;
                    await semWait.WaitAsync(); // wait until finished changing
                }
            }
            else semWait.Release();
        }

        private void mySlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (!dontChangeFlag)
            {
                dontChangeFlag = true;
                mySlider.Value = 50;
                // below process continues after the second   
                // mySlider_ValueChanged has finished its job
                dontChangeFlag = false;
            }
        }
    }
}